<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use App\Models\User;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = '/';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'first_name' => ['required', 'string', 'min:2', 'max:20'],
            'last_name' => ['required', 'string', 'min:2', 'max:20'],
            'phone' => ['required', 'numeric', 'digits_between:8,16','unique:users'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:3', 'confirmed'],
        ],
                [
                    'email.unique' => 'The email address is taken',
                ]);

               
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {
        $checkUser = User::count();
        
        if ($checkUser === 0) {
              $ran = substr(str_shuffle("123456789abcdefghijklmnopqrstuvwxyz"), 0, 4);
        $slug = $data['first_name'] . $ran;
        return User::create([
            'first_name' => $data['first_name'],
            'last_name' => $data['last_name'],
            'phone' => $data['phone'],
            'email' => $data['email'],
            'role' => 'admin',
            'user_status' => 'Active',
            'username' => $slug,
            'picture' => 'rabbit.jpg',
            'password' => Hash::make($data['password']),
        ]);
        }
        else{
                $ran = substr(str_shuffle("123456789abcdefghijklmnopqrstuvwxyz"), 0, 4);
        $slug = $data['first_name'] . $ran;
        return User::create([
            'first_name' => $data['first_name'],
            'last_name' => $data['last_name'],
            'phone' => $data['phone'],
            'email' => $data['email'],
            'role' => 'user',
            'user_status' => 'Active',
            'username' => $slug,
            'picture' => 'rabbit.jpg',
            'password' => Hash::make($data['password']),
        ]);
        }
        
    
      
    }
    
}
